package context;

import java.io.File;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;

import model.SettlementModel;
import output.Outputter;
import output.SettlementOutputter;

import com.vividsolutions.jts.geom.Point;

import dataLoad.ShapeFileLoader;
import entities.Settlement;
import entities.SettlementEdge;
import repast.simphony.context.Context;
import repast.simphony.context.space.graph.NetworkFactoryFinder;
import repast.simphony.context.space.grid.GridFactoryFinder;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunState;
import repast.simphony.parameter.Parameters;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.graph.Network;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridBuilderParameters;
import repast.simphony.space.grid.RandomGridAdder;
import repast.simphony.space.grid.StickyBorders;


/**
 * Class to instantiate the simulation.
 * 
 * @author 
 *
 * @param <T>
 */
public class ContextCreator<T> implements ContextBuilder<T>{
	
	/**List containing simulation settlements **/
	public static List<Settlement> cities=new ArrayList<Settlement>();
	
	/**Network used for settlement connections**/
	private Network<Settlement> cNetwork;
	
	/**
	 * Context for building and instantiating the simulation
	 */
	public Context build(Context<T> context) {
		ContextCreator.cities.clear();
		cNetwork = 
				(Network<Settlement>) NetworkFactoryFinder.createNetworkFactory(null).
				createNetwork("Flow",context,true);
		
		
		Outputter.runNumber=RunState.getInstance().getRunInfo().getRunNumber();
		Outputter.started=false;
		Parameters p = RunEnvironment.getInstance().getParameters();
		
		String loc = p.getValueAsString("scenarioData");
		double constant=p.getDouble("constant");
		double epsilon1=p.getDouble("epsilon1");
		double endTick=((Double)p.getValue("endTick"));
		
		SettlementModel.Constant=constant;
		SettlementModel.Epsilon1=epsilon1;
		
		createSettlement(loc,context,p);
		createNetworkCities();
		
		RunEnvironment.getInstance().endAt(endTick);
		
		File file = new File("./output/");
		file.mkdir();
		
		return context;
	}
	
	/**
	 * Method for city creation.
	 * @param location the location of the data file
	 * @param context the simulation context that contains simulation data
	 */
	public void createSettlement(String location,Context context,Parameters par){
		 cities.clear();
		 
		 
		 List<Map<String,Object>> data=ShapeFileLoader.shpLoader(new File(location));
		 Iterator<Map<String,Object>> in = data.iterator();
		 List<Integer>numbers=new ArrayList<Integer>();
		
		 Map mm = in.next();
		 Iterator<String>ki = mm.keySet().iterator();
		 while(ki.hasNext()){
			 String key = ki.next();
			 if(key.contains("point")){
				 String[] ss = key.split("point");
				 int nn=Integer.parseInt(ss[1]);
				 numbers.add(nn);
			 }
		 }
		 
		 int population=par.getInteger("population");
		 double alpha=((Double)par.getValue("alpha")).doubleValue();
		 double beta=((Double)par.getValue("beta")).doubleValue();
		 
		 double nn=438;
		 double count=0;
		 for(Map m : data){
			 double d = RandomHelper.nextDouble();
			 
		     
			 Settlement c = new Settlement();
			 Point p=(Point) m.get("the_geom");
			 int id=-1;
			 
			 try{
				 id=(Integer)m.get("cat");
			 }
			 catch(java.lang.ClassCastException e){
				 id=((Long)m.get("cat")).intValue();
			 }
			
			 String name=null;
			 try{
				 name=(String)m.get("Site_name");
			 }
			 catch(Exception e){
				 name=(String)m.get("NAME");
			 }
			 if(name==null){
				 try{
					 name=((Long)m.get("NUMBER")).toString();
				 }
				 catch(ClassCastException e){
					 name=((Double)m.get("NUMBER")).toString();
				 }
			 }
			 double eS=(Double)m.get("size");
		
			 double eP=(Double)m.get("scalar");
			 double rS=0;
			 try{
				 rS=(Double)m.get("Size__ha_");
			 }
			 catch(NullPointerException e){
				 rS=(Double)m.get("SIZE__HA_");
			 }
			 int rid=-1;
			 try{
				 rid=(Integer)m.get("cat");
			 }
			 catch(java.lang.ClassCastException e){
				 rid=((Long)m.get("cat")).intValue();
			 }
			 
			 double x=p.getX();
			 double y=p.getY();
			
			 
			for(int nw: numbers){
				 double v=(Double)m.get("point"+nw);
				 c.getTravelCosts().put(nw, v);
			 }
			 
			 c.setRealSize(rS);
			 c.setPopulation(population);
			 c.setPoint(p);
			 c.setX(x);
			 c.setY(y);
			 if(name!=null && !name.equals(""))
				 c.setName(name);
			 else
				 c.setName(Integer.toString(rid));
			 c.setIdN(id);
			 c.setSize(eS);
			 c.setScalar(eP);
			 
			 
			 c.setAlpha(alpha);
			 c.setBeta(beta);
			 
			 cities.add(c);
		 }
		 context.addAll(cities);
		 SettlementOutputter co = new SettlementOutputter();
		 context.add(co);	
	}
	
	/**
	 * The network creator for sites in study area. Each settlement will be connected to all other settlements.
	 */
	public void createNetworkCities(){
		
		for(int ii=0; ii < cities.size();ii++){
			Settlement c = cities.get(ii);
			for(int i=0; i < cities.size(); i++){
				Settlement cc = cities.get(i);
				SettlementEdge cef = new SettlementEdge(c,cc);
				cef.setWeight(c.getTravelCosts().get(cc.idN));
				cNetwork.addEdge(cef);
			}
		}
	}

}
