package entities;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import model.SettlementModel;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.engine.schedule.ScheduledMethod;




import com.vividsolutions.jts.geom.Point;

/**
 * The City class.
 * 
 * @author
 *
 */
public class Settlement {

	//identifiers (spatial or by id)
	/**The point location of the settlement*/
	public Point point;
	
	/**The name of the settlement*/
	public String name;
	
	/**The identifier number*/
	public int idN;
	
	/**Number of people in the settlement"**/
	public int population;
	
	/**Variable not used except in analysis. This is the population of the settlement in simulations*/
	public int numberSize;
	
	/**The standard deviation of the settlement population*/
	public double numberSize_sd;
	
	/**The x location*/
	public double x;
	
	/**The y location*/
	public double y;
	
	//input values
	
	/**The size of the settlement as determined by the input file*/
	public double size;
	
	/**The alpha value input*/
	public double alpha;
	
	/**The beta value input*/
	public double beta;
	
	/**The person benefit multiplier; this is used in the instantiation process to create the benefit multiplier for people*/
	public double benefitMultiplier;
	
	/**This is used in the instantiation process to create the benefit multiplier standard deviation (if used)*/
	public double benefitMultiplier_sd;
	
	/**The is the endogenous or exogenous social-environmental benefits given to a settlement*/
	public double settlementBenefit;
	
	/**Travel cost values from the cost surface inputs*/
	public Map<Integer,Double>travelCosts=new HashMap<Integer,Double>();
	
	//calculated values
	/**The flow value to a settlement*/
	public double flow;
	
	/**The standard deviation (used in analysis) at a given settlement*/
	public double flow_sd;
	
	/**adjusting values size usign a scalar*/
	public double scalar; 
	
	/**real size as determined by survey or empirical data*/
	public double realSize;
	
	/**The settlement model used*/
	public SettlementModel model=new SettlementModel();;
	
	public double getX() {
		return x;
	}
	public void setX(double x) {
		this.x = x;
	}
	public double getY() {
		return y;
	}
	public void setY(double y) {
		this.y = y;
	}
	
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	public int getIdN() {
		return idN;
	}
	public void setIdN(int idN) {
		this.idN = idN;
	}
	public Map<Integer, Double> getTravelCosts() {
		return travelCosts;
	}
	public void setTravelCosts(Map<Integer, Double> travelCosts) {
		this.travelCosts = travelCosts;
	}
	public double getSize() {
		return size;
	}
	public void setSize(double size) {
		this.size = size;
	}
	public double getAlpha() {
		return alpha;
	}
	public void setAlpha(double alpha) {
		this.alpha = alpha;
	}
	public double getBeta() {
		return beta;
	}
	public void setBeta(double beta) {
		this.beta = beta;
	}
	
	public Point getPoint() {
		return point;
	}
	public void setPoint(Point point) {
		this.point = point;
	}
	public double getFlow() {
		return flow;
	}
	public void setFlow(double flow) {
		this.flow = flow;
	}
	
	public double getBenefitMultiplier() {
		return benefitMultiplier;
	}
	public void setBenefitMultiplier(double benefitMultiplier) {
		this.benefitMultiplier = benefitMultiplier;
	}
	public SettlementModel getModel() {
		return model;
	}
	public void setModel(SettlementModel model) {
		this.model = model;
	}
		
	public int getNumberSize() {
		return numberSize;
	}
	public void setNumberSize(int numberSize) {
		this.numberSize = numberSize;
	}
	
	public double getNumberSize_sd() {
		return numberSize_sd;
	}
	public void setNumberSize_sd(double numberSize_sd) {
		this.numberSize_sd = numberSize_sd;
	}
	public double getFlow_sd() {
		return flow_sd;
	}
	public void setFlow_sd(double flow_sd) {
		this.flow_sd = flow_sd;
	}
	
	public double getBenefitMultiplier_sd() {
		return benefitMultiplier_sd;
	}
	public void setBenefitMultiplier_sd(double benefitMultiplier_sd) {
		this.benefitMultiplier_sd = benefitMultiplier_sd;
	}
	
	
	public double getSettlementBenefit() {
		return settlementBenefit;
	}
	public void setSettlementBenefit(double settlementBenefit) {
		this.settlementBenefit = settlementBenefit;
	}
	
	public int getPopulation() {
		return population;
	}
	public void setPopulation(int population) {
		this.population = population;
	}
	
	public double getScalar() {
		return scalar;
	}
	public void setScalar(double scalar) {
		this.scalar = scalar;
	}
	
	
	
	public double getRealSize() {
		return realSize;
	}
	public void setRealSize(double realSize) {
		this.realSize = realSize;
	}
	/**
	 * Method to run the model from main entity object.
	 */
	@ScheduledMethod(start = 1, interval=1, pick=ScheduledMethod.ALL, priority = ScheduleParameters.FIRST_PRIORITY)
	public void stepOne(){
		if(model==null)
			model=new SettlementModel();
		
		model.run(this);
	}
	
	@ScheduledMethod(start = 1, interval=1, pick=ScheduledMethod.ALL, priority = ScheduleParameters.FIRST_PRIORITY-1)
	public void stepTwo(){
		model.calculateSize(this);
	}
	
}
