package model;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import context.ContextCreator;
import entities.Settlement;
import entities.SettlementEdge;
import repast.simphony.engine.environment.RunState;
import repast.simphony.space.graph.Network;


/**
 * Basic Entropy-Maximization Wilson model based on UCL Grant work.
 * 
 * @author 
 *
 */
public class SettlementModel {
	
	Map<String,Settlement>flowEdges=new HashMap<String,Settlement>();
	static Map<Double,Double>tickValue=new HashMap<Double,Double>(); //tick value and total size value for sites.

	public static double Constant=0;;
	public static double Epsilon1=0;
	
	/**
	 * Run the flow method from here.
	 * @param c
	 */
	public void run(Settlement c) {
		flow(c);
		
	}
	
	/**
	 * Method to calculate flow using standard entropy maximization calculation.
	 * 
	 * See: Altaweel, M. (2013; http://discovery.ucl.ac.uk/1412092/). 
	 * The application of an entropy maximising model for understanding the 
	 * rise of urbanism. In T. J. Wilkinson, M. Gibson, J. Christiansen, M. Widell (Eds.), 
	 * Models of Mesopotamian Landscapes: How Small-Scale Processes Contribute to the Growth of Early Civilizations.
	 * 
	 * Equation #1 in publication.
	 * 
	 * 
	 * 
	 * @param c
	 */
	public void flow(Settlement c) {
	//	Context context=RunState.getSafeMasterContext();
		
		Iterator<Settlement> ci=ContextCreator.cities.iterator();
		Network flowN = (Network) RunState.getInstance().getMasterContext().getProjection("Flow");

		double betaCal=totalFlow(c);
		while(ci.hasNext()){
			Settlement c2 = ci.next();
			
			double distance=0;
			if(c2==c ||flowN.getEdge(c,c2).getWeight()==0.0){
				distance=0;
			}
			else{
				double d=flowN.getEdge(c,c2).getWeight();
				distance=Math.log(d);
			}
	
			double alphaCal=Math.pow(c2.getSize(),c2.getAlpha());
			double alphaB=Math.exp(-c2.getBeta()*distance);
			
			double numDe=(alphaCal*alphaB)/betaCal;
			
			double flow=c.getScalar()*(double)c.getPopulation()*numDe;
		
			((SettlementEdge)flowN.getEdge(c,c2)).setUsed(flow+((SettlementEdge)flowN.getEdge(c,c2)).getUsed());
			c2.setFlow(flow+(c2.getFlow()));
		}
//		System.out.println(c.getFlow());
	}
	
	/**
	 * Method to calculate Settlement size.
	 * 
	 * See Altaweel (2013; http://discovery.ucl.ac.uk/1412092/) equation #3
	 * 
	 * @param c a Settlement object.
	 */
	public void calculateSize(Settlement c){
		double revenue=c.getFlow();
		double size=c.getSize()+Epsilon1*(revenue-(Constant*c.getSize()));
		c.setSize(size);
	}
	
	/**
	 * This is done as needed for total flow of cities. This is the denominator used in equation #1, 
	 * which calculates total entropy for settlements based on a given settlement c.
	 * 
	 * @param c a Settlement object.
	 * 
	 * @return the total flow
	 */
	public double totalFlow(Settlement c){
		double totalSize=0;
		Network flow = (Network) RunState.getInstance().getMasterContext().getProjection("Flow");
		Iterator<Settlement> ic = ContextCreator.cities.iterator();
		while(ic.hasNext()){
			Settlement c2 = ic.next();
			double distance=0;
			if(c2==c ||flow.getEdge(c,c2).getWeight()==0.0)
				distance=0;
			else
				distance=Math.log(flow.getEdge(c,c2).getWeight());
			
			totalSize+=Math.pow(c2.getSize(),c2.alpha)*Math.exp(-c2.beta*distance);
		}
		return totalSize;
	}
}
