package output;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.geotools.data.DataUtilities;
import org.geotools.data.simple.SimpleFeatureCollection;
import org.geotools.feature.FeatureCollection;
import org.geotools.feature.FeatureCollections;
import org.geotools.feature.IllegalAttributeException;
import org.geotools.feature.SchemaException;
import org.geotools.feature.simple.SimpleFeatureBuilder;
import org.geotools.feature.simple.SimpleFeatureTypeBuilder;
import org.geotools.referencing.CRS;
import org.geotools.referencing.ReferencingFactoryFinder;
import org.geotools.referencing.operation.DefiningConversion;
import org.opengis.feature.simple.SimpleFeature;
import org.opengis.feature.simple.SimpleFeatureType;
import org.opengis.parameter.ParameterValueGroup;
import org.opengis.referencing.FactoryException;
import org.opengis.referencing.NoSuchAuthorityCodeException;
import org.opengis.referencing.NoSuchIdentifierException;
import org.opengis.referencing.crs.CRSAuthorityFactory;
import org.opengis.referencing.crs.CRSFactory;
import org.opengis.referencing.crs.CoordinateReferenceSystem;
import org.opengis.referencing.crs.GeographicCRS;
import org.opengis.referencing.crs.ProjectedCRS;
import org.opengis.referencing.cs.CartesianCS;
import org.opengis.referencing.operation.Conversion;
import org.opengis.referencing.operation.MathTransformFactory;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.MultiLineString;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.PrecisionModel;

import entities.Settlement;
import entities.SettlementEdge;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunState;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.space.graph.Network;
import repast.simphony.space.graph.RepastEdge;

	
/**
 * Class used to output model behaviour and results
 * 
 * @author 
 *
 */
	public class SettlementOutputter {
		
		/**
		 * This method calls the data outputter for text file output.
		 */
		@ScheduledMethod(start = 100, interval=1,pick=ScheduledMethod.ALL, priority = 2)
		public void settlementOutput(){
			Iterator<Settlement> ci=
					RunState.getInstance().getMasterContext().getAgentLayer(Settlement.class).iterator();
		
			List<String> output=new ArrayList<String>();
			while(ci.hasNext()){
				Settlement c = ci.next();
				output.addAll(Outputter.doOutput(c));
			}
			Outputter.writeData(output);
		}
		
		/**
		 * This method calls the shapefile point data output as needed based on time specified here.
		 */
		@ScheduledMethod(start = 100, pick=ScheduledMethod.ALL, priority = 2)
		public void doOutput(){
			Iterator<Settlement> ci=
					RunState.getInstance().getMasterContext().getAgentLayer(Settlement.class).iterator();
		
		
			Iterator<SettlementEdge> cii =  
					((Network) RunState.getInstance().getMasterContext().getProjection("Flow")).getEdges().iterator();
			
			List<Settlement> cities = new ArrayList<Settlement>();
			List<SettlementEdge> canals = new ArrayList<SettlementEdge>();
			while(ci.hasNext()){
				cities.add((Settlement) ci.next());
			}
		
			writeShapefileRepeat(cities);
			writeNetwork();	
		}
		
		@ScheduledMethod(start = 1, interval=1,pick=ScheduledMethod.ALL, priority = ScheduleParameters.LAST_PRIORITY)
		public void recalculatePopulation(){
			Iterator<Settlement> ci=
					RunState.getInstance().getMasterContext().getAgentLayer(Settlement.class).iterator();
			List<Settlement> cities = new ArrayList<Settlement>();
			double totalFlow=0;
			int totalPopulation=0;
			while(ci.hasNext()){
				Settlement c = ci.next();
				totalFlow+=c.getFlow();
				totalPopulation+=c.getPopulation();
			}
			ci=RunState.getInstance().getMasterContext().getAgentLayer(Settlement.class).iterator();
			while(ci.hasNext()){
				Settlement c = ci.next();
				
				c.setPopulation((int)Math.rint((double)totalPopulation*(c.getFlow()/totalFlow)));
	//			c.setFlow(0.0);
			}
		}		
	
		/**
		 * This calls the shapefile writer to create output.
		 * @param cities
		 */
		public void writeShapefileRepeat(List<Settlement> cities){
			double tick=RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
			try {
				writeShapefile(cities, "settlement"+Math.rint(tick));
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		
		/**
		 * This is the Nystuen-Dacey graph algorithm and line creator.
		 */
		public void writeNetwork(){
			double tick=RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
			Network<Settlement> flowN = (Network) RunState.getInstance().getMasterContext().getProjection("Flow");
			Iterator<Settlement> ic = RunState.getInstance().getMasterContext().getObjects(Settlement.class).iterator();
			List<SimpleFeature>collection=new ArrayList<SimpleFeature>();
			SimpleFeatureType featureType=null;
			CoordinateReferenceSystem crs=null;
			CRSAuthorityFactory   factory = CRS.getAuthorityFactory(true);
			
			try {
				crs = factory.createCoordinateReferenceSystem("EPSG:32638");
			} catch (NoSuchAuthorityCodeException e3) {
				// TODO Auto-generated catch block
				e3.printStackTrace();
			} catch (FactoryException e3) {
				// TODO Auto-generated catch block
				e3.printStackTrace();
			}
			try {
	            featureType = DataUtilities.createType("ConnectionOutput", 
	            		"geom:MultiLineString,Name1:java.lang.String,Name2:java.lang.String,Alpha:java.lang.Double,"
	            		+ "Beta:java.lang.Double,Flow:java.lang.Double");
	            featureType=SimpleFeatureTypeBuilder.retype(featureType,returnCRS());
            } catch (SchemaException e2) {
	            // TODO Auto-generated catch block
	            e2.printStackTrace();
            }
            
			Iterator icc = flowN.getEdges().iterator();
			double aveFlowCost=0;
			double graphLineLength=0;
			double cc=0;
			double ll=0;
			int id=0;
			
			while(icc.hasNext()){
				SettlementEdge e = (SettlementEdge)icc.next();
				Settlement c1 =(Settlement) e.getSource();
				
				Settlement c2 = (Settlement)e.getTarget();
				if(c2==c1)
					continue;
				
				SettlementEdge e2=(SettlementEdge)flowN.getEdge(c2,c1);
				aveFlowCost=+e.getWeight();
				cc++;
				
				if(e.getUsed()>e2.getUsed() && e.getUsed()>0){
					String n1=Integer.toString(c1.idN);
					String n2=Integer.toString(c2.idN);
				
					try {
						graphLineLength+=lineCreation(((Settlement)e.getSource()).getPoint(),((Settlement)e.getTarget()).getPoint(),id,
								collection,featureType,n1,n2,c1.alpha,c1.beta,e.getUsed());
						ll++;
						id++;
					} catch (SchemaException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					} catch (IllegalAttributeException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
				}
			}
			FeatureCollection fCollection=DataUtilities.collection(collection);
			
			aveFlowCost=aveFlowCost/cc;  //average flow and distance ratio
			graphLineLength=graphLineLength/ll;  //average Nystuen-Dacey line length
			System.out.println("Average Line Length: "+graphLineLength);
			ShapefileWriter writer = new ShapefileWriter();
			int n=RunState.getInstance().getRunInfo().getRunNumber();
			File shpFile = new File("./output/","ConnectionOutput"+tick+"_"+n+".shp");
			writer.write(shpFile, fCollection, featureType);
		}
		
		/**
		 * Method to write the shapefile output.
		 * @param poly
		 * @param category
		 * @throws Exception
		 */
		  public static void writeShapefile(List<Settlement> poly,String category) throws Exception {
			  	int n=RunState.getInstance().getRunInfo().getRunNumber();
			    File shpFile = new File("./output/","SettlementOutput"+category+"_"+n+".shp");	
			    
			    CoordinateReferenceSystem crs=null;
				 CRSAuthorityFactory   factory = CRS.getAuthorityFactory(true);
					try {
						crs = factory.createCoordinateReferenceSystem("EPSG:32638");
				} catch (NoSuchAuthorityCodeException e3) {
					// TODO Auto-generated catch block
					e3.printStackTrace();
				} catch (FactoryException e3) {
					// TODO Auto-generated catch block
					e3.printStackTrace();
				}
				
				ShapefileWriter writer = new ShapefileWriter();
				SimpleFeatureType featureType=DataUtilities.createType("SettlementOutput", 
						"geom:Point,Tick:java.lang.Double,Name:java.lang.String,Flow:java.lang.Double,Size:java.lang.Double,Population:java.lang.Integer," +
						"Alpha:java.lang.Double,Beta:java.lang.Double");
				featureType=SimpleFeatureTypeBuilder.retype(featureType,returnCRS());
				FeatureCollection collection = createCollection(poly,featureType);
				writer.write(shpFile,collection,featureType);
			   
			 }
		  		 
		  /**
		   * Method to create a feature collection of structure objects 
		   * @param Citys the structures
		   * @param featureType the feature type containing the attributes for a shapefile
		   * @return a feature collection
		   * @throws Exception an exception with the data
		   */
		public static FeatureCollection createCollection(List<Settlement> Citys, SimpleFeatureType
				featureType) throws Exception{
			
			 double tick = RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
			Iterator<Settlement> ic = Citys.iterator();
			List<SimpleFeature>fType=new ArrayList<SimpleFeature>();
			int id=0;
			while(ic.hasNext()){
				  Settlement f = ic.next();
				  GeometryFactory ff = new GeometryFactory();
				  Point p=ff.createPoint(new Coordinate(f.getX(),f.getY(),0));
				  String name=Integer.toString(f.idN);
				  double alpha=f.getAlpha();
				  double beta=f.getBeta();
				  double flow=f.getFlow();
				  double size=f.getRealSize();
				  int population=f.getPopulation();
				  
				  CoordinateReferenceSystem crs=null;
				  CRSAuthorityFactory   factory = CRS.getAuthorityFactory(true);
					try {
						crs = factory.createCoordinateReferenceSystem("EPSG:32638");
					} catch (NoSuchAuthorityCodeException e3) {
						// TODO Auto-generated catch block
						e3.printStackTrace();
					} catch (FactoryException e3) {
						// TODO Auto-generated catch block
						e3.printStackTrace();
					}
					
				  featureType=SimpleFeatureTypeBuilder.retype(featureType,returnCRS());
				  SimpleFeatureBuilder featureBuilder = new SimpleFeatureBuilder(featureType);
				  SimpleFeature feature = featureBuilder.buildFeature(Integer.toString(id),new Object[] {p,tick,name,flow,
					  size,population,alpha,beta});
				  fType.add( feature );
				  id++;
			  }
			 FeatureCollection collection=DataUtilities.collection(fType);
			 return collection;
		}
		
		/**
		 * The CRS file needed for shapefile output.
		 * @return
		 */
		private static CoordinateReferenceSystem returnCRS(){
			MathTransformFactory mtFactory = ReferencingFactoryFinder.getMathTransformFactory(null);
		    CRSFactory crsFactory = ReferencingFactoryFinder.getCRSFactory(null);
		    
		    GeographicCRS geoCRS = org.geotools.referencing.crs.DefaultGeographicCRS.WGS84;
		    CartesianCS cartCS = org.geotools.referencing.cs.DefaultCartesianCS.GENERIC_2D;
		    
		    ParameterValueGroup parameters = null;
			try {
				parameters = mtFactory.getDefaultParameters("Transverse_Mercator");
			} catch (NoSuchIdentifierException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		    parameters.parameter("central_meridian").setValue(-111.0);
		    parameters.parameter("latitude_of_origin").setValue(0.0);
		    parameters.parameter("scale_factor").setValue(0.9996);
		    parameters.parameter("false_easting").setValue(500000.0);
		    parameters.parameter("false_northing").setValue(0.0);
		    Conversion conversion = new DefiningConversion("Transverse_Mercator", parameters);
		    
		    Map<String, ?> properties = Collections.singletonMap("name", RunEnvironment.getInstance().getParameters().getString("locationData"));
		    ProjectedCRS projCRS=null;
			try {
				projCRS = crsFactory.createProjectedCRS(properties, geoCRS, conversion, cartCS);
			} catch (FactoryException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		
		    return (CoordinateReferenceSystem)projCRS;
		}
		
		/**
		 * Method to create shapefile line points.
		 * @param p1
		 * @param p2
		 * @param id
		 * @param collection
		 * @param featureType
		 * @param name1
		 * @param name2
		 * @param alpha
		 * @param beta
		 * @param used
		 * @return
		 * @throws SchemaException
		 * @throws IllegalAttributeException
		 */
		public static double lineCreation(Point p1,Point p2,int id,List<SimpleFeature> collection,SimpleFeatureType featureType,
				String name1,String name2, double alpha,double beta,double used) 
				throws SchemaException, IllegalAttributeException{
			ShapefileWriter writer = new ShapefileWriter();
			Coordinate[] cs = {p1.getCoordinate(),p2.getCoordinate()};
			
			LineString l = new LineString(cs,new PrecisionModel(),1);
			MultiLineString ls = new MultiLineString(new LineString[]{l},new PrecisionModel(),1);
			
			SimpleFeatureBuilder featureBuilder = new SimpleFeatureBuilder(featureType);
			
			SimpleFeature feature = featureBuilder.buildFeature(Integer.toString(id),new Object[] {ls,name1,name2,alpha,beta,used});
			collection.add( feature );
			return ls.getLength();
		}
	}

