package model;

import static org.junit.Assert.assertTrue;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.junit.Test;

import context.ContextCreator;
import entities.Settlement;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunState;
import repast.simphony.parameter.ParametersCreator;
import repast.simphony.space.graph.Network;


/**
 * Test class to look at methods in urban model
 * @author 
 *
 */
public class TestSettlementModel {

	// necessary to cities to be tested and their functions
		static {
			ParametersCreator creator = new ParametersCreator();
			creator.addParameter("epsilon", double.class, 1.0, false);
			RunEnvironment.init(null, null, creator.createParameters(), false);
		}
		
		
		/**
		 * Create a settlement
		 */
		public void createSettlement(){
			
			Map<Integer,Double> costs1=new HashMap<Integer,Double>();
			Map<Integer,Double> costs2=new HashMap<Integer,Double>();
			costs1.put(0, 1000.0);
			costs1.put(1, 50000.0);
			costs2.put(0, 5000.5);
			costs2.put(1, 1000.0);
			
			
			for(int i=0; i < 2; i++) {
				Settlement s = new Settlement();
				s.setAlpha(1.0);
				s.setBeta(1.0);
				s.setFlow(1.0);
				s.setIdN(1);
				s.setModel(new SettlementModel());
				s.setName("name");
				s.setSize(1.0);
				if(i==0)
					s.setTravelCosts(costs1);
				else
					s.setTravelCosts(costs2);
				ContextCreator.cities.add(s);
			}
		}
		/**
		 * Test the main entropy method to see if it works properly()
		 */
		@Test
		public void testModel(){
			SettlementModel acm = new SettlementModel();
			createSettlement();
			Iterator<Settlement>si=ContextCreator.cities.iterator();
			Map<Integer,Integer>numbs=new HashMap<Integer,Integer>();
			
			Settlement c = ContextCreator.cities.get(0);
			double betaCal=totalFlow(c);
			int i=0;
			
			while(si.hasNext()){
				Settlement c2 = si.next();
				double distance=0;
				if(c==c2){
					distance=0;
				}
				else
					distance=Math.log(c2.getTravelCosts().get(i));
		
				double alphaCal=Math.pow(c2.getSize(),c2.getAlpha())*Math.exp(-c2.getBeta()*distance);
				
				double numDe=alphaCal/betaCal;
				
				System.out.println(numDe);
				
				assertTrue(numDe<1.0);
				
				i++;
			}
			
		}
		
		/**
		 * Calculate total flow, which is used in the entropy method
		 * @param c
		 * @return
		 */
		public double totalFlow(Settlement c){
			double totalSize=0;
			Iterator<Settlement> ic = ContextCreator.cities.iterator();
			int i=0;
			while(ic.hasNext()){
				Settlement c2 = ic.next();
				double distance=0;
				if(c==c2){
					distance=0;
				}
				else
					distance=Math.log(c2.getTravelCosts().get(i));
				
				totalSize+=Math.pow(c2.getSize(),c2.alpha)*Math.exp(-c2.beta*distance);
				i++;
			}
			return totalSize;
		}
		
		@Test
		public void testCalculateSize(){
			Settlement c = new Settlement();
			c.setFlow(114.0);
			c.setSize(1.0);
			SettlementModel.Constant=1.0;
			SettlementModel.Epsilon1=0.5;
			
			double revenue=c.getFlow();
			double size=c.getSize()+SettlementModel.Epsilon1*(revenue-(SettlementModel.Constant*c.getSize()));
			assertTrue(size==57.5);
		}
		
		/**
		 * Test net flows
		 */
		@Test
		public void testNetFlow(){
			SettlementModel acm = new SettlementModel();
			createSettlement();
			Iterator<Settlement>si=ContextCreator.cities.iterator();
			while(si.hasNext()){
				Settlement s = si.next();
				assertTrue(s.flow==1.0);
			}
			
		}
}
